/*******************************************************************************
  Copyright(c) 2000 - 2003 Radu Corlan. All rights reserved.
  
  This program is free software; you can redistribute it and/or modify it 
  under the terms of the GNU General Public License as published by the Free 
  Software Foundation; either version 2 of the License, or (at your option) 
  any later version.
  
  This program is distributed in the hope that it will be useful, but WITHOUT 
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or 
  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for 
  more details.
  
  You should have received a copy of the GNU General Public License along with
  this program; if not, write to the Free Software Foundation, Inc., 59 
  Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  
  The full GNU General Public License is included in this distribution in the
  file called LICENSE.
  
  Contact Information: radu@corlan.net
*******************************************************************************/

/* do a photometry run; mostly ugly glue code between the gui_star_list, libccd
   aperture_photometry and photsolve.c */

#define _GNU_SOURCE
#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <string.h>
#include <stdlib.h>
#include <libgen.h>
#include <math.h>
#include <ctype.h>

#include <gdk/gdkkeysyms.h>
#include <gtk/gtk.h>

#include "gcx.h"
#include "catalogs.h"
#include "gui.h"
#include "interface.h"
#include "params.h"
#include "sourcesdraw.h"
#include "photsolve.h"
#include "obsdata.h"

#define INDENT 4 

static double frame_scint(struct ccd_frame *fr, struct wcs *wcs);


static inline void put_indent(int indent, int indstart, FILE *fp)
{
	int i;
	for (i = 0; i < indent * INDENT - indstart; i++)
		fputc(' ', fp);
}

static inline double distance(double x1, double y1, double x2, double y2)
{
	return sqrt(sqr(x1 - x2) + sqr(y1 - y2));
}

static void center_phot_stars(gpointer window, struct gui_star_list *gsl, 
			      struct ccd_frame *fr, double max_ce)
{
	GSList *apsl=NULL, *sl;
	struct gui_star *gs;
	struct star st;
	int all, centered = 0;
	double err = 0.0;

	apsl = gui_stars_of_type(gsl, TYPE_MASK_PHOT);
	all = g_slist_length(apsl);
	if (all == 0) {
		err_printf_sb2(window, "No phot stars\n");
		return;
	}
//	d3_printf("center stars got %d\n", all);
	sl = apsl;
	while (sl != NULL) {
		double d;
		gs = GUI_STAR(sl->data);
		sl = g_slist_next(sl);
		if (gs->x < 0 || gs->x > fr->w || gs->y < 0 || gs->y > fr->h)
			continue;
		d1_printf("Star near %.1f %.1f ", gs->x, gs->y);
		get_star_near(fr, (int)(round(gs->x)), (int)(round(gs->y)), 0, &st);
		d1_printf("found at %.1f %.1f ", st.x, st.y) ;
		if (((d = distance(gs->x, gs->y, st.x, st.y)) > max_ce)) {
			d1_printf(" -- too far, skipped\n");
			if (gs->s != NULL) {
				CAT_STAR(gs->s)->flags |= CPHOT_NOT_FOUND;
				CAT_STAR(gs->s)->flags &= ~CPHOT_CENTERED;
			}
		} else {
			if (gs->s != NULL) {
				CAT_STAR(gs->s)->flags &= ~CPHOT_NOT_FOUND;
				CAT_STAR(gs->s)->flags |= CPHOT_CENTERED;
			}
			err += sqr(d);
			gs->x = st.x;	
			gs->y = st.y;
			d1_printf(" -- centered\n");
			centered ++;
		}
	}
	info_printf_sb2(window, "photometry", 5000, 
			"%d of %d stars centered, rms error = %.2f pix\n", 
			centered, all, sqrt(err / centered));
	g_slist_free(apsl);
	gtk_widget_queue_draw(window);
}

/* initialise ap_params from user parameters */
static void ap_params_from_par(struct ap_params *ap)
{
	ap->r1 = P_DBL(AP_R1);
	ap->r2 = P_DBL(AP_R2);
	ap->r3 = P_DBL(AP_R3);
	ap->quads = ALLQUADS;
	ap->sky_method = P_INT(AP_SKY_METHOD);
	ap->sigmas = P_DBL(AP_SIGMAS);
	ap->max_iter = P_INT(AP_MAX_ITER);
	ap->sat_limit = P_DBL(AP_SATURATION);
	ap->std_err = 0.0;
}

static void aphot_stars(GSList *sl, struct ccd_frame *fr, struct bad_pix_map *bpmap)
{
//	struct star *star;
	struct gui_star *gs;
	struct cat_star *cats;
	struct ap_params ap;
	int ret;

	ap_params_from_par(&ap);
	while (sl != NULL) {
		gs = GUI_STAR(sl->data);
		sl = g_slist_next(sl);
		if (gs->s != NULL) 
			cats = CAT_STAR(gs->s);
		else
			continue;
		if (cats->data != NULL) {
			ret = aperture_photometry(fr, cats->data, &ap, bpmap);
			if (ret < 0) { /* remove the star from the cats so we know 
					* we had an error measuring it */
				free(cats->data);
				cats->data = NULL;
			} 
			
		}
	}
}

#define REPORT_NOISE 1
#define REPORT_AP_PARAMS 1

static void rep_ap_params(FILE *repfp, int indent)
{
	char **c;
	int i;

	put_indent(indent, 0, repfp);
	fprintf(repfp, "ap_par (r1 %.0f r2 %.0f r3 %.0f",
		P_DBL(AP_R1), P_DBL(AP_R2), P_DBL(AP_R3));

	c = PAR(AP_SKY_METHOD)->choices;
	if (c != NULL) {
		for (i = 0; i < P_INT(AP_SKY_METHOD) && *c != NULL; i++)
			c++;
		if (*c != NULL) {
			fprintf(repfp, " sky_method \"%s\" sigmas %.1f",
				*c, P_DBL(AP_SIGMAS) );
		}
	}
	fprintf(repfp, ")\n");
}

static void rep_noise_params(struct ccd_frame *fr, struct wcs *wcs, FILE *repfp, int indent)
{
	double scint;
	put_indent(indent, 0, repfp);
	fprintf(repfp, "noise (read %.1f eladu %.1f flat %.1f",
		fr->exp.rdnoise, fr->exp.scale, fr->exp.flat_noise);
	if ((scint = frame_scint(fr, wcs)) != 0.0)
		fprintf(repfp, " scint %.3f)\n", scint);
	else
		fprintf(repfp, ")\n");
}

static void rep_obs(struct ccd_frame *fr, struct wcs *wcs, 
		    FILE *repfp, int indent)
{
	double v;
	char s[80];
	int fields = 0;

	put_indent(indent, 2, repfp);
	fprintf(repfp, "( ");
	if (fits_get_string(fr, P_STR(FN_OBJECT), s, 79) > 0) {
		fprintf(repfp, "object \"%s\" ", s);
		fields++;
	}
	if (fits_get_string(fr, P_STR(FN_OBJCTRA), s, 79) > 0) {
		fprintf(repfp, "ra \"%s\" ", s);
		fields++;
	}
	if (fits_get_string(fr, P_STR(FN_OBJCTDEC), s, 79) > 0) {
		fprintf(repfp, "dec \"%s\" ", s);
		fields++;
	}
	if (fits_get_double(fr, P_STR(FN_EQUINOX), &v) > 0) {
		fprintf(repfp, "equinox %.0f ", v);
		fields++;
	} 
	if (fields > 0) {
		fprintf(repfp, "\n");
		put_indent(indent, 0, repfp);
		fields = 0;
	}
	if (fits_get_double(fr, P_STR(FN_MJD), &v) > 0) {
		fprintf(repfp, "mjd %.6f ", v);
		fields++;
	} else if (fits_get_double(fr, P_STR(FN_JDATE), &v) > 0) {
		fprintf(repfp, "mjd %.6f ", v - 2400000.5);
		fields++;
	} 

	if (fits_get_string(fr, P_STR(FN_TELESCOP), s, 79) > 0) {
		fprintf(repfp, "telescope \"%s\" ", s);
		fields++;
	}
	if (fits_get_double(fr, P_STR(FN_APERTURE), &v) > 0) {
		fprintf(repfp, "aperture %.1f ", v);
		fields++;
	}
	if (fields > 0) {
		fprintf(repfp, "\n");
		put_indent(indent, 0, repfp);
		fields = 0;
	}
	if (fits_get_double(fr, P_STR(FN_EXPTIME), &v) > 0) {
		fprintf(repfp, "exptime %.2f ", v);
		fields++;
	}
	if (fits_get_double(fr, P_STR(FN_SNSTEMP), &v) > 0) {
		fprintf(repfp, "sns_temp %.1f ", v);
		fields++;
	}
	if (fits_get_string(fr, P_STR(FN_FILTER), s, 79) > 0) {
		fprintf(repfp, "filter \"%s\" ", s);
		fields++;
	}
	if (fields > 0) {
		fprintf(repfp, "\n");
		put_indent(indent, 0, repfp);
		fields = 0;
	}
	if (fits_get_string(fr, P_STR(FN_LATITUDE), s, 79) > 0) {
		fprintf(repfp, "latitude \"%s\" ", s);
		fields++;
	}
	if (fits_get_string(fr, P_STR(FN_LONGITUDE), s, 79) > 0) {
		fprintf(repfp, "longitude \"%s\" ", s);
		fields++;
	}
	if (fits_get_double(fr, P_STR(FN_ALTITUDE), &v) > 0) {
		fprintf(repfp, "altitude %.0f ", v);
		fields++;
	}
	if (fits_get_double(fr, P_STR(FN_AIRMASS), &v) > 0) {
		fprintf(repfp, "airmass %.3f ", v);
		fields++;
	} else { /* attempt to calculate airmass from other fields */
		v = frame_airmass(fr, wcs->xref, wcs->yref);
		if (v > 0.0) {
			fprintf(repfp, "airmass %.3f ", v);
			fields++;
		}
	}

	if (fields > 0) {
		fprintf(repfp, "\n");
		put_indent(indent, 0, repfp);
		fields = 0;
	}
	if (fits_get_string(fr, P_STR(FN_OBSERVER), s, 79) > 0)
		fprintf(repfp, "observer \"%s\" ", s);
	fprintf(repfp, ")\n");
}

static void rep_stars_start(FILE *repfp, int indent)
{
	put_indent(indent, 0, repfp);
	fprintf(repfp, "stars (\n");
}
static void rep_stars_end(FILE *repfp, int indent)
{
	put_indent(indent, 0, repfp);
	fprintf(repfp, ")\n");
}

/* returns the indent level for further alist members */
static int frame_rep_start(struct ccd_frame *fr, struct wcs *wcs,
			   FILE *repfp, struct bad_pix_map *bpmap)
{
	int indent = 0;

	if (repfp == NULL)
		return 0;

	fprintf(repfp, "(");
	put_indent(1, 1, repfp);
	fprintf(repfp, "observation\n");
	indent ++;
	rep_obs(fr, wcs, repfp, indent+1);
	if (REPORT_NOISE) {
		rep_noise_params(fr, wcs, repfp, indent);
	}
	if (REPORT_AP_PARAMS) {
		rep_ap_params(repfp, indent);
	}
	return indent;
}

static void frame_rep_end(struct ccd_frame *fr, FILE *repfp)
{
	if (repfp == NULL)
		return;

	fprintf(repfp, ")\n\n");
} 

/* set the band names we use in this reduction */
static void set_band_names(struct band_names *bnames, char *band)
{
	if (band != NULL && band[0] != 0 && !P_INT(AP_FORCE_IBAND)) {
		strncpy(bnames->iband, band, BAND_NAME_SIZE);
		strncpy(bnames->sband, band, BAND_NAME_SIZE);
	} else {
		strncpy(bnames->iband, P_STR(AP_IBAND_NAME), BAND_NAME_SIZE);
		strncpy(bnames->sband, P_STR(AP_IBAND_NAME), BAND_NAME_SIZE);
	}
//	strncpy(bnames->color, P_STR(AP_COLOR_NAME), BAND_NAME_SIZE);
}

static void rep_star_noise(struct star *s, FILE *repfp, int indent)
{
	put_indent(indent, 0, repfp);
	fprintf(repfp, "noise (sky %.3f read %.3f photon %.3f scint %.3f)",
		s->aph.sky_err * s->aph.star_all / s->aph.star,
		s->aph.rd_noise / s->aph.star,
		s->aph.pshot_noise / s->aph.star ,
		s->aph.scint);
}

static void rep_star_flags(struct cat_star *cats, FILE *repfp, int indent)
{
	int i=0, flags;

	flags = cats->flags & (CPHOT_MASK | CAT_VARIABLE | CAT_ASTROMET);
	put_indent(indent, 0, repfp);
	fprintf(repfp, "flags ( ");
	while (cat_flag_names[i] != NULL) {
		if (flags & 0x01)
			fprintf(repfp, "%s ", cat_flag_names[i]);
		flags >>= 1;
		i++;
	}
	fprintf(repfp, ")\n");
}

static void rep_std_stars(GSList *sl, struct pfit_star* stds, 
			  FILE *repfp, int indent)
{
	int i = 0;
	struct cat_star *cats;
	char decs[64];
	char ras[64];

	while (sl != NULL) {
		cats = CAT_STAR(sl->data);
		sl = g_slist_next(sl);
		degrees_to_dms_pr(ras, cats->ra / 15.0, 2);
		degrees_to_dms_pr(decs, cats->dec, 1);
		put_indent(indent, 2, repfp);
		fprintf(repfp, "( name \"%s\" type std ra \"%s\" dec \"%s\"\n", 
			cats->name, ras, decs);
		if (cats->comments != NULL) {
			put_indent(indent, 0, repfp);
			fprintf(repfp, "comments \"%s\"\n", cats->comments);
		}
		put_indent(indent, 0, repfp);
		fprintf(repfp, "smags \"%s\"\n", cats->smags);
		put_indent(indent, 0, repfp);
		fprintf(repfp, "imags \"%s\"\n", cats->imags);
		put_indent(indent, 0, repfp);
		fprintf(repfp, "residual %.3f weight %.3f\n", stds[i].residual, stds[i].weight);
		rep_star_flags(cats, repfp, indent);
		if (REPORT_NOISE) 
			rep_star_noise(cats->data, repfp, indent);
		fprintf(repfp, " )\n");
		i++;
	}

}

static void rep_pgm_stars(GSList *sl, struct pfit_star* pgms, 
			  FILE *repfp, int indent)
{
	int i = 0;
	struct cat_star *cats;
	char decs[64];
	char ras[64];

	while (sl != NULL) {
		cats = CAT_STAR(sl->data);
		sl = g_slist_next(sl);
		put_indent(indent, 2, repfp);
		degrees_to_dms_pr(ras, cats->ra / 15.0, 2);
		degrees_to_dms_pr(decs, cats->dec, 1);
		fprintf(repfp, "( name \"%s\" type target ra \"%s\" dec \"%s\"\n", 
			cats->name, ras, decs);
		if (cats->comments != NULL) {
			put_indent(indent, 0, repfp);
			fprintf(repfp, "comments \"%s\"\n", cats->comments);
		}
		put_indent(indent, 0, repfp);
		fprintf(repfp, "smags \"%s\"\n", cats->smags);
		put_indent(indent, 0, repfp);
		fprintf(repfp, "imags \"%s\"\n", cats->imags);
		rep_star_flags(cats, repfp, indent);
		if (REPORT_NOISE) 
			rep_star_noise(cats->data, repfp, indent);
		fprintf(repfp, " )\n");
		i++;
	}

}

static void rep_transform(struct star_transform *trans, double meu, FILE *repfp, int indent)
{
	if ((trans->flags & TRANSFORM_ZEROP_VALID) == 0)
		return;
	put_indent(indent, 0, repfp);
	fprintf(repfp, "transform (zerop %.3f zp_err %.3f ",
		trans->zerop, sqrt(trans->cov_zz));
	if (trans->flags & TRANSFORM_COLTERM_VALID) {
		fprintf(repfp, "colc %.3f cc_err %.3f ",
			trans->colterm, sqrt(trans->cov_cc));
	}
	fprintf(repfp, "me1 %.2f)\n", meu);
}

/* gather star data into pfit format. Return -1 if essential info cannot be found
 * relies on instrumental mags being already pushed to the imags in cats
 */
static int std_cats_to_pfit(struct pfit_star *std, struct cat_star *cats, 
			     struct band_names *bnames)
{
	double m=0.0, e=99.0;
	int ret;
	struct star *s;

	s = cats->data;
	std->imag = s->aph.absmag;
	std->imag_err = sqrt(sqr(s->aph.magerr) + sqr(s->aph.scint));
	std->flags = (PFIT_IMAG_VALID | PFIT_IMAGERR_VALID);

	ret = get_band_by_name(cats->smags, bnames->sband, &m, &e);
	d3_printf("%s [%s]=%.3f\n", cats->name, bnames->sband, m);
	if (ret) {
		err_printf("Cannot get %s band magnitude of %s (%s)\n", 
			   bnames->sband, cats->name, cats->smags);
		std->smag = 0.0;
		std->smag_err = 0.0;
		std->color = 0.0;
		std->color_err = 0.0;
		return -1;
	} else {
		if (e > 90.0) {
			cats->flags |= CPHOT_UNDEF_ERR;
			std->smag_err = P_DBL(AP_DEFAULT_STD_ERROR);
		} else {
			std->smag_err = e;
			std->flags |= PFIT_SMAGERR_VALID;
		}
		std->smag = m;
		std->flags |= PFIT_SMAG_VALID;
	}
	e = 99.0;
	std->color = 0.0;
	std->color_err = 0.0;
//	cats->flags |= CPHOT_NO_COLOR;
	return 0;
}

static int pgm_cats_to_pfit(struct pfit_star *pgm, struct cat_star *cats, 
			     struct band_names *bnames)
{
	double e;
	struct star *s;

	s = cats->data;
	pgm->imag = s->aph.absmag;
	pgm->imag_err = sqrt(sqr(s->aph.magerr) + sqr(s->aph.scint));
	if (s->aph.flags & AP_STAR_SKIP)
		cats->flags |= CPHOT_BADPIX;
	pgm->flags = (PFIT_IMAG_VALID | PFIT_IMAGERR_VALID);

	e = 99.0;
	pgm->color = 0.0;
	pgm->color_err = 0.0;
//	cats->flags |= CPHOT_NO_COLOR;
	return 0;
}


/* fit stars measured by aphot_stars to a response slope, to determine
 * the zero points of the transformation, and report the stars
 * for now, we do a robust least-squares without taking colors into account. 
 * That could be done at a later stage, to combine more observations */
static void fit_report_stars(GSList *sl, FILE *repfp, char * band, int indent)
{
	struct gui_star *gs;
	struct cat_star *cats;
	struct pfit_star *stds = NULL, *pgms = NULL;
	int all, i;
	struct band_names bnames;
	struct star_transform trans;
	double meu;
	struct star *s;
	GSList *std=NULL, *pgm=NULL;
	int nstd = 0, npgm = 0;

	trans.flags = 0;

	all = g_slist_length(sl);
	set_band_names(&bnames, band);

/* load the stars into std and pgm, and push the instrumental magnitudes into the list */
	while (sl != NULL) {
		gs = GUI_STAR(sl->data);
		sl = g_slist_next(sl);
		if (gs->s != NULL) 
			cats = CAT_STAR(gs->s);
		else
			continue;
		if (cats->data == NULL)
			continue;

		s = cats->data;

		if (s->aph.flags & (AP_STAR_SKIP | AP_BPSTAR))
			cats->flags |= CPHOT_BADPIX;
		if (s->aph.flags & AP_BURNOUT)
			cats->flags |= CPHOT_BURNED;
		update_band_by_name(&cats->imags, bnames.iband, 
				    s->aph.absmag, 
				    sqrt(sqr(s->aph.magerr) + sqr(s->aph.scint)));

		if (CATS_TYPE(cats) == CAT_STAR_TYPE_APSTAR) {
			pgm = g_slist_prepend(pgm, cats);
			npgm++;
		}
		if (CATS_TYPE(cats) == CAT_STAR_TYPE_APSTD) {
			std = g_slist_prepend(std, cats);
			nstd++;
		}
	}
	if (nstd == 0) 
		goto exit;

/* convert stars to pfit_star */

	if (nstd != 0) {
		sl = std;
		stds = calloc(nstd, sizeof(struct pfit_star));
		i = 0;
		while (sl != NULL) {
			cats = CAT_STAR(sl->data);
			sl = g_slist_next(sl);
			std_cats_to_pfit(&(stds[i]), cats, &bnames);
			i++;
		}
	}
	if (npgm != 0) {
		sl = pgm;
		pgms = calloc(npgm, sizeof(struct pfit_star));
		i = 0;
		while (sl != NULL) {
			cats = CAT_STAR(sl->data);
			sl = g_slist_next(sl);
			pgm_cats_to_pfit(&(pgms[i]), cats, &bnames);
			i++;
		}
	}

/* do the fit */
	fit_std_stars(stds, nstd, &trans, &meu);
	transform_stars(pgms, npgm, &trans);
	if (npgm != 0 && bnames.iband != NULL) { /* update the mag strings */
		sl = pgm;
		i = 0;
		while (sl != NULL) {
			cats = CAT_STAR(sl->data);
			sl = g_slist_next(sl);
			if ((bnames.sband != NULL) && (pgms[i].flags & PFIT_SMAG_VALID)) {
				update_band_by_name(&cats->smags, bnames.sband, 
						    pgms[i].smag, pgms[i].smag_err);
			}
			i++;
		}
	}
	rep_stars_start(repfp, indent);
	rep_std_stars(std, stds, repfp, indent+1);
	rep_pgm_stars(pgm, pgms, repfp, indent+1);
	rep_stars_end(repfp, indent);
	rep_transform(&trans, meu, repfp, indent);

	free(pgms);
	free(stds);

exit:
	g_slist_free(pgm);
	g_slist_free(std);
}

/* calculate scintillation for frame */
static double frame_scint(struct ccd_frame *fr, struct wcs *wcs)
{
	double r;
	double t, d, am;
	int lack = 0;

// determine exposure time

	if (fits_get_double(fr, P_STR(FN_EXPTIME), &t) <= 0 ) {
		err_printf("bad exposure time, assuming 1 sec\n");
		t = 10.0;
		lack ++;
	}
// telescope aperture
	if ((fits_get_double(fr, P_STR(FN_APERTURE), &r) > 0)) {
		d = r;
	} else {
		d = P_DBL(OBS_APERTURE);
		d1_printf("Taking aperture from OBS, rather than the file header\n");
	}
	if ((fits_get_double(fr, P_STR(FN_AIRMASS), &r) > 0)) {
		am = r;
	} else {
		am = frame_airmass(fr, wcs->xref, wcs->yref);
		if (am == 0) {
//		err_printf("could not determine airmass, assuming 1.5\n");
			am = 1.5;
			lack ++;
		}
	}
	if (lack < 2) /* we tolerate one missing param */
		return scintillation(t, d, am);
	else 
		return 0.0;
}

/* single-field photmetric run. Will write the report on repfp if non-null
 * assumes the stars have already been centered */
static char * run_phot(gpointer window, struct wcs *wcs, struct gui_star_list *gsl, 
		       struct ccd_frame *fr, FILE *repfp)
{
	GSList *apsl=NULL, *sl;
	int all, indent;
	struct star *star;
	struct gui_star *gs;
	struct cat_star *cats;
	struct bad_pix_map *bpmap;
	char *ret; int rp=0;
	double scint;
	char s[80];

	bpmap = gtk_object_get_data(GTK_OBJECT(window), "bad_pix_map");

	apsl = gui_stars_of_type(gsl, TYPE_MASK_PHOT);
	all = g_slist_length(apsl);
	if (all == 0) {
		err_printf_sb2(window, "No phot stars\n");
		return  NULL;
	}
	scint = frame_scint(fr, wcs);

/* attach struct star's to the cats of the list */
	sl = apsl;
	while (sl != NULL) {
		gs = GUI_STAR(sl->data);
		sl = g_slist_next(sl);
		if (gs->s != NULL) 
			cats = CAT_STAR(gs->s);
		else
			continue;
		star = calloc(1, sizeof(struct star));
		star->x = gs->x;
		star->y = gs->y;
		star->aph.scint = scint;
		cats->data = star;
	}
/* do the actual photometry */
	aphot_stars(apsl, fr, bpmap);
	d3_printf("aphotted\n");
	indent = frame_rep_start(fr, wcs, repfp, bpmap);
	if (fits_get_string(fr, P_STR(FN_FILTER), s, 79) > 0) {
		fit_report_stars(apsl, repfp, s, indent);
	} else {
		d1_printf("Frame does not have filter info, using default\n");
			fit_report_stars(apsl, repfp, NULL, indent);
	}
	frame_rep_end(fr, repfp);
	fflush(repfp);
/* now print 'short' report and release the stars we attached */
	ret = calloc(81, 1);
	sl = apsl;
	while (sl != NULL) {
//		d3_printf("releasing\n");
		gs = GUI_STAR(sl->data);
		sl = g_slist_next(sl);
		if (gs->s != NULL) 
			cats = CAT_STAR(gs->s);
		else
			continue;
		if ((CATS_TYPE(cats)== CAT_STAR_TYPE_APSTAR) && (cats->smags != NULL) && rp < 80) {
			rp += snprintf(ret+rp, 80-rp, "%s:%s ", cats->name, cats->smags);
		}
		if (cats->data != NULL) {
			free(cats->data);
			cats->data = NULL;
		}
	}
//	d3_printf("releasing list\n");
	g_slist_free(apsl);
	return ret;
}

/* photometry callback from menu; report goes to stdout */
void photometry_cb(gpointer window, guint action, GtkWidget *menu_item)
{
	struct gui_star_list *gsl = NULL;
	struct image_channel *i_chan = NULL;
	struct ccd_frame *fr = NULL;
	struct wcs *wcs;
	char *ret;

	i_chan = gtk_object_get_data(GTK_OBJECT(window), "i_channel");
	if (i_chan == NULL || i_chan->fr == NULL) {
		err_printf_sb2(window, "No frame\n");
		return;
	} else {
		fr = i_chan->fr;
	}
	gsl = gtk_object_get_data(GTK_OBJECT(window), "gui_star_list");
	if (gsl == NULL) {
		err_printf_sb2(window, "No stars\n");
		return;
	} 
	wcs = gtk_object_get_data(GTK_OBJECT(window), "wcs_of_window");
	if ((wcs == NULL) || ((wcs->wcsset & WCS_VALID) == 0)) {
		err_printf_sb2(window, "Invalid wcs\n");
		return;
	} 

	d3_printf("airmass %f\n", frame_airmass(fr, wcs->xref, wcs->yref));

	switch(action) {
	case PHOT_CENTER_STARS:
		center_phot_stars(window, gsl, fr, P_DBL(AP_MAX_CENTER_ERR));
		break;
	case PHOT_RUN:
		center_phot_stars(window, gsl, fr, P_DBL(AP_MAX_CENTER_ERR));
		ret = run_phot(window, wcs, gsl, fr, stdout);
		if (ret != NULL) {
			statusbar2_message(window, ret, "phot_result", 10000);
			free(ret);
		}
		break;
	default:
		err_printf("unknown action %d in photometry_cb\n", action);
	}
}

/* run photometry on the stars in window, writing report to fd 
 * a 'short' result (malloced string) is returned (NULL for an error) */
char * phot_to_fd(gpointer window, FILE *fd)
{
	struct gui_star_list *gsl = NULL;
	struct image_channel *i_chan = NULL;
	struct ccd_frame *fr = NULL;
	struct wcs *wcs;

	i_chan = gtk_object_get_data(GTK_OBJECT(window), "i_channel");
	if (i_chan == NULL || i_chan->fr == NULL) {
		err_printf("No frame\n");
		return NULL;
	} else {
		fr = i_chan->fr;
	}
	gsl = gtk_object_get_data(GTK_OBJECT(window), "gui_star_list");
	if (gsl == NULL) {
		err_printf("No phot stars\n");
		return NULL;
	} 
	wcs = gtk_object_get_data(GTK_OBJECT(window), "wcs_of_window");
	if ((wcs == NULL) || ((wcs->wcsset & WCS_VALID) == 0)) {
		err_printf("Invalid wcs\n");
		return NULL;
	} 

	center_phot_stars(window, gsl, fr, P_DBL(AP_MAX_CENTER_ERR));
	return run_phot(window, wcs, gsl, fr, fd);

}

